#include "DistrhoPlugin.hpp"
#include "synth.h"

START_NAMESPACE_DISTRHO

class TabPlugin : public Plugin
{
public:
	TabPlugin()
		: Plugin(kParameterCount, 0, 0),
		sampleRate(getSampleRate()),
		synth(sampleRate),
		fParameters { 0 }
	{
	}

protected:
	const char* getLabel() const override { return "yaw-shepard"; }
	const char* getDescription() const override { return "Generalized Shepard tone, tablet UI"; }
	const char* getMaker() const override { return "yaw-audio"; }
	const char* getHomePage() const override { return "https://yaw.man/plugins/yaw-shepard"; }
	const char* getLicense() const override { return "Fuck You Pay Me"; }
	uint32_t    getVersion() const override { return d_version(1, 0, 0); }
	int64_t     getUniqueId() const override { return d_cconst('y', 's', 'p', 'd'); }


	void initParameter(uint32_t index, Parameter& parameter) override
	{
		parameter.hints = kParameterIsAutomable;
		parameter.ranges.def = 0.0f;
		parameter.ranges.min = 0.0f;
		parameter.ranges.max = 1.0f;

		switch (index)
		{
		case kFundamentalFrequency:
			parameter.name = "hz";
			parameter.symbol = "hz";
			parameter.hints = kParameterIsOutput;
			parameter.ranges.def = 55.0f;
			parameter.ranges.min = 55.0f;
			parameter.ranges.max = 880.0f;
		case ktpax:
			parameter.name = "x";
			parameter.symbol = "x";
			break;
		case ktpay:
			parameter.name = "y";
			parameter.symbol = "y";
			break;
		case ktpaz:
			parameter.name = "z";
			parameter.symbol = "z";
			break;
		case ktpap:
			parameter.name = "p";
			parameter.symbol = "p";
			break;
		case kParameterButtonA:
			parameter.name = "Button A";
			parameter.symbol = "A";
			parameter.hints |= kParameterIsBoolean;
			break;
		case kParameterButtonB:
			parameter.name = "Button B";
			parameter.symbol = "B";
			parameter.hints |= kParameterIsBoolean;
			break;
		case kParameterTime:
			parameter.name = "t";
			parameter.symbol = "t";
			parameter.hints = kParameterIsOutput;
			break;
		}
	}

	void sampleRateChanged(double newRate) override
	{
		synth.setSampleRate(sampleRate, newRate);
		sampleRate = newRate;
	}

	float getParameterValue(uint32_t index) const override
	{
		return fParameters[index];
	}

	void setParameterValue(uint32_t idx, float val) override
	{
		fParameters[idx] = val;
		switch (idx) {
		case ktpax:
			synth.setPhaseVelocity(val);
			break;
		case ktpay:
			synth.setTimbre(val);
			break;
		case ktpaz:
			break;
		case ktpap:
			synth.setVolume(val);
			break;
		case kParameterButtonA:
			break;
		case kParameterButtonB:
			break;
		}
	}

	void run(const float** inputs, float** outputs, uint32_t frames) override
	{
		synth.process(*outputs, frames);
		parity = !parity;
		fParameters[kParameterTime] += parity ? 1 : -1;
		fParameters[kFundamentalFrequency] = synth.hzFund;
	}

private:
	float fParameters[kParameterCount];
	double sampleRate;
	bool parity = false;

	Synth synth;
	DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(TabPlugin)
};

Plugin* createPlugin()
{
	return new TabPlugin();
}

END_NAMESPACE_DISTRHO
