#include "DistrhoPlugin.hpp"
#include "Resampler.hpp"
#include "scale/scale.h"

START_NAMESPACE_DISTRHO

class PitchCorrector : public Plugin
{
public:
	PitchCorrector()
		: Plugin(1, 0, 0),
		  resamplers{},
		  detectors{}
	{
	}

protected:
	const char *getLabel() const override { return "yaw-totune"; }
	const char *getDescription() const override { return "Pitch corrector"; }
	const char *getMaker() const override { return "yaw-audio"; }
	const char *getHomePage() const override { return "https://yaw.man/plugins/yaw-totune"; }
	const char *getLicense() const override { return "Fuck you pay me"; }
	uint32_t getVersion() const override { return d_version(1, 0, 0); }
	int64_t getUniqueId() const override { return d_cconst('y', 't', 't', 'n'); }

	void initParameter(uint32_t index, Parameter &parameter) override
	{
		parameter.hints = kParameterIsAutomable;
		parameter.ranges.def = 0.0f;
		parameter.ranges.min = 0.0f;
		parameter.ranges.max = 1000.0f;
		parameter.name = "param";
		parameter.symbol = "param";
	}

	void sampleRateChanged(double newRate) override
	{
		hzNyq = newRate / 2;
	}

	float getParameterValue(uint32_t index) const override
	{
		return period;
	}

	void setParameterValue(uint32_t idx, float val) override
	{
	}

	double debugTone(float* output, uint32_t frames, float period, float phase)
	{
		for(uint i = 0; i < frames; ++i) {
			*output = 0.5 * sinf(	M_PI * 2.0 * phase	);
			phase += 1.f / period;
			phase -= (int)phase;
			++output;
		};

		return phase;
	}

	void run(const float **inputs, float **outputs, uint32_t frames) override
	{
		for (int chn = 0; chn < 2; ++chn)
		{
			resamplers[chn].write(inputs[chn], frames);
			detectors[chn].downsample(inputs[chn], frames);
			auto detectedPitches = detectors[chn].detectPitch(frames);
			double taux = 1.0;
			period = 1023.0;
			if (detectedPitches[0])
			{
				period = resamplers[chn].getAccuratePeriod(detectedPitches[0], detectedPitches[1]);
				double correctPeriod = scale.getNearestPeriod(period);
				if( correctPeriod > 1.0 ) taux = period / correctPeriod;
			};
			resamplers[chn].resample(outputs[chn], frames, 0.95, 109);
		}
	}

private:
	double period = 1023.0;
	double hzNyq;
	std::array<Resampler<float>, 2> resamplers;
	std::array<Detector<float>, 2> detectors;
	std::array<double, 2> debugPhases = {0};
	Scale scale{440.0};
	DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(PitchCorrector)
};

Plugin *createPlugin()
{
	return new PitchCorrector();
}

END_NAMESPACE_DISTRHO
